require 'rails_helper'

module Nukleus
  RSpec.describe DocumentsController, type: :controller do
    before(:example) do
        NoBrainer.purge!
        NoBrainer.sync_indexes
    end

    let(:document) { Document.new({:document_id =>"test_doc_id", :tags => ["a","b"], :meta_data => { test: "me" }}) }

    # describe "print routes" do
    #   it "test" do
    #     test_routes = []

    #     Nukleus::Engine.routes.routes.each do |route|
    #       route = route.path.spec.to_s
    #       test_routes << route
    #     end
    #     puts test_routes
    #   end
    # end

    describe "GET #index" do
      routes { ::Nukleus::Engine.routes }
      render_views

      it "returns http success" do
        get :index, :format => :json
        expect(response).to have_http_status(:success)
      end
    end

    describe "GET #show" do
      routes { ::Nukleus::Engine.routes }
      render_views

      before(:each) do
        FactoryGirl.create(:nukleus_document)
      end

      it "returns http success" do
        get :show, id: "factory_doc_id", :format => :json
        expect(response).to have_http_status(:success)
      end
    end

    describe "POST #create" do
      routes { ::Nukleus::Engine.routes }
      render_views

      it "returns http success using json" do
        expect {
            post :create, document: { document_id: "test", tags: ["bla", "blubb"], meta_data: { test: "me", follow: "you"} }, :format => :json
        }.to change { Document.count }
        expect(response).to have_http_status(:created)

        resp = JSON.parse(response.body)

        # TODO: Make this possible with new structure: expect(resp["token"]).not_to eq(nil)
        expect(resp["data"]).not_to eq(nil)
      end

    end

    describe "PUT #update" do
      routes { ::Nukleus::Engine.routes }

      before(:each) do
        FactoryGirl.create(:nukleus_document)
      end

      it "returns http success" do
        put :update, id: "factory_doc_id", document: { document_id: "test", tags: ["bla", "blubb"], meta_data: {test: "me", follow: "you"} }, :format => :json 
        expect(response).to have_http_status(:success)
      end
    end

    describe "PATCH #update" do
      routes { ::Nukleus::Engine.routes }

      before(:each) do
        FactoryGirl.create(:nukleus_document)
      end

      it "returns http success" do
        patch :update, id: "factory_doc_id", document: { document_id: "test", tags: ["bla", "blubb"], meta_data: {test: "me", follow: "you"} }, :format => :json 
        expect(response).to have_http_status(:success)
      end
    end

    describe "delete #destroy" do
      routes { ::Nukleus::Engine.routes }

      before(:each) do
        FactoryGirl.create(:nukleus_document)
      end

      it "returns http success" do
        delete :destroy, id: "factory_doc_id", :format => :json
        expect(response).to have_http_status(:success)
      end
    end

  end
end
